#include "attractiveforcelocator.h"
#include "math.h"

#include <QPoint>
#ifdef _DEV_
#include <QTextStream>
#endif


AttractiveForceLocator::AttractiveForceLocator(ProximityDataModel& model,
                                               int timeout)
    : ProximityLocator(model, timeout)
{
}

AttractiveForceLocator::~AttractiveForceLocator()
{
}

/*!
  Diese Methode wird immer wieder ausgeführt und stellt die Implementierung
  des virtuellen Slots ProximityLocator::calculate() dar.
  In diesem Fall werden die Signalstärken zu den einzelnen Antennen ausgewertet,
  um die Position der einzelnen Tags zu kalkulieren.
  Komplexität ist n*m mit n - Anzahl der Tags und m - Anzahl der Antennen
  */
void AttractiveForceLocator::calculate() {
    foreach(PTNP::LocatableProximityTag* tag, _model.tags()) {
        if(!tag->fixed()) {
            QPoint entireMovement = getMovement(*tag);
            QPoint tagPos(tag->pos().x, tag->pos().y);
            tagPos+= entireMovement;
#ifdef _DEV_
            QTextStream cout(stdout, QIODevice::WriteOnly);
            cout<<"afl.cpp: tagid: "<<tag->oid()<<" bewegt sich um: "<<sqrt(pow(entireMovement.x(), 2)+pow(entireMovement.y(), 2))<<endl;
#endif
            PTNP::position newPos;
            newPos.f = tag->pos().f;
            newPos.x = tagPos.x();
            newPos.y = tagPos.y();
            tag->setPosition(newPos);
#ifdef _DEV_
            cout<<"afl.cpp: tagid: "<<tag->oid()<<" x: "<<tag->pos().x<<" y:"<<tag->pos().y<<" floor:"<< tag->pos().f<<"\n"<<flush;
#endif
        }
        emit sendPositionMessage(tag->oid(), tag->pos());
    }
}
