#include "contactforcelocator.h"

#include "math.h"
#include <QPointF>
#include <QDataStream>
#include <QTextStream>


ContactForceLocator::ContactForceLocator(ProximityDataModel& model,
                                               int timeout)
    : ProximityLocator(model, timeout)
{
}

ContactForceLocator::~ContactForceLocator()
{
}

/*!
  Diese Methode wird immer wieder ausgeführt und stellt die Implementierung
  des virtuellen Slots ProximityLocator::calculate() dar.
  */
void ContactForceLocator::calculate() {
    foreach(PTNP::LocatableProximityTag* tag, _model.tags()) {
        if(!tag->fixed()) {
            PTNP::position tagPos = tag->pos();
            QMap<PTNP::aid, quint16> antennaAttractions = tag->getAntennaAttractions(); // keine Distanz - leere Liste behandeln
            QMap<PTNP::oid, quint16> tagAttractions = tag->getFixedTagAttractions();

            QList<QPoint> attractionvectors;
            QPoint movementgesamt;
            QPoint tagpos(tagPos.x, tagPos.y);
            // zu allen Antennen die Vektoren berechnen - mit der Länge aus getDistances()

#ifdef _DEV_
            QTextStream cout(stdout, QIODevice::WriteOnly);
            cout<<"cfl.cpp: anziehungskraefte: ";
#endif
            foreach(PTNP::aid id, antennaAttractions.uniqueKeys()) {
                PTNP::ProximityAntenna* a = _model.antennas().value(id);
                QPoint antpos(a->pos().x, a->pos().y);
                QPoint delta = antpos - tagpos;
                delta *= ((antennaAttractions.value(id)/_tagweight)/100);
                attractionvectors.append(delta);
#ifdef _DEV_
                cout<<" id: "<<id<<" ";
                cout<<antennaAttractions.value(id);
                cout<<" ";
#endif
                movementgesamt +=delta;
            }
            foreach(PTNP::oid id, tagAttractions.uniqueKeys()) {
                PTNP::LocatableProximityTag* t = _model.tags().value(id);
                QPoint fixedPos(t->pos().x, t->pos().y);
                QPoint delta = fixedPos - tagpos;
                delta *= ((tagAttractions.value(id)/_tagweight)/100);
                attractionvectors.append(delta);
                movementgesamt += delta;
            }

#ifdef _DEV_
            cout<<"\n";
#endif
            movementgesamt/= attractionvectors.length();// auch hier ist das wohl nicht so gut... (siehe attractiveforcelocator)

            tagpos+=movementgesamt; // neu setzten, dann evaluieren

            // Wenn die Anziehungskraft zu einer Antenne größer ist, als zu einem
            // kontaktiertem Tag, dieses aber weiter weg von der Antenne steht, so muss
            // die Berechnung zum kontaktiertem Tag hin korregiert werden.
            // oder die Position gar an diesem Tag gespiegelt werden (!?)

            foreach(PTNP::oid tagId, tagAttractions.uniqueKeys()) {
                PTNP::LocatableProximityTag* t = _model.tags().value(tagId);
                QPoint contactedTagPosition(t->pos().x, t->pos().y);
                foreach(PTNP::aid antId, antennaAttractions.uniqueKeys()) {
                    PTNP::ProximityAntenna* a = _model.antennas().value(antId);
                    QPoint antpos(a->pos().x, a->pos().y);
                    QPoint deltaContactedTagAnt =  contactedTagPosition - antpos;
                    QPoint deltaTagAnt = tagpos - antpos;
                    if(sqrt(pow(deltaContactedTagAnt.x(),2)+pow(deltaContactedTagAnt.y(),2))
                        < sqrt(pow(deltaTagAnt.x(),2)+pow(deltaTagAnt.y(),2))) {
//                        QPoint deltaTagToTag = contactedTagPosition - tagpos;
//                        tagpos = tagpos + deltaTagToTag;
//                        contactedTagPosition = contactedTagPosition - deltaTagToTag
                        QPoint temp = tagpos;
                        tagpos = contactedTagPosition;
                        contactedTagPosition = temp;
                        PTNP::position p = t->pos();
                        p.x = contactedTagPosition.x();
                        p.y = contactedTagPosition.y();
                        t->setPosition(p);
                    }
                }
            }
            tagPos.x = tagpos.x();
            tagPos.y = tagpos.y();
            tag->setPosition(tagPos);
#ifdef _DEV_
            cout<<"cfl.cpp tagid: "<<tag->oid()<<" x: "<<tag->pos().x<<" y:"<<tag->pos().y<<" floor:"<< tag->pos().f<<"\n"<<flush;
#endif

        }
        emit sendPositionMessage(tag->oid(), tag->pos());
    }
}
