#include "landmarclocator.h"

#include "math.h"
#include <QPoint>
#include <QPair>
#include <QSettings>
#include <QTextStream>

struct referenceListEntry {
    PTNP::oid id;
    QPoint delta;
};

qreal length(QPoint a) {
    return sqrt(pow(a.x(), 2)+pow(a.y(), 2));
}

bool operator < (const referenceListEntry &e1, const referenceListEntry &e2) {
    return sqrt(pow(e1.delta.x(), 2)+pow(e1.delta.y(), 2)) < sqrt(pow(e2.delta.x(), 2)+pow(e2.delta.y(), 2));
}

LANDMARCLocator::LANDMARCLocator(ProximityDataModel& model,
                                 int timeout)
    : ProximityLocator(model, timeout)
{
    QSettings settings;
    _consideredTags = settings.value("consideredtags", "4").toUInt();
#ifdef _DEV_
    QTextStream cout(stdout, QIODevice::WriteOnly);
    cout<<"landmarclocator.cpp: consideredtags: "<<_consideredTags<<endl;
#endif
    if(_consideredTags>_model.fixedTags().size()) {
        QTextStream cerr(stderr, QIODevice::WriteOnly);
        cerr<<"Number in Settings for consideredtags ("<<_consideredTags<<") ";
        cerr<<"was higher than declared number of referencetags ("<<_model.fixedTags().size()<<")."<<endl;
        cerr<<"I'm taking "<<_model.fixedTags().size()<<" as number of consideredtags."<<endl;
        _consideredTags = _model.fixedTags().size();
    }
}

LANDMARCLocator::~LANDMARCLocator()
{
}

void LANDMARCLocator::calculate()
{
    QMap<PTNP::oid, QPoint> compareMap;
    foreach(PTNP::LocatableProximityTag* tag, _model.fixedTags()) {
        compareMap[tag->oid()] = getMovement(*tag); // den gesamtVektor in die Vergleichsmap eintragen
    }
    foreach(PTNP::LocatableProximityTag* tag, _model.tags()) {
        if(!tag->fixed()) {
            QPoint tagMovement = getMovement(*tag);
            QList<referenceListEntry> referenceTagDeltaList;
            foreach(PTNP::LocatableProximityTag* fixedTag, _model.fixedTags()) {
                referenceListEntry entry;
                entry.delta = compareMap[fixedTag->oid()] - tagMovement;
                entry.id = fixedTag->oid();
                referenceTagDeltaList.append(entry);
            }
            qSort(referenceTagDeltaList.begin(), referenceTagDeltaList.end());
            if(_consideredTags>0) {
                for(int i=0; i<_consideredTags; i++) {
                    PTNP::LocatableProximityTag* fixedTag = _model.fixedTags().value(referenceTagDeltaList.at(i).id);
                    QPoint deltaSignalStrength = referenceTagDeltaList.at(i).delta;
                    QPoint fixedTagPosition = QPoint(fixedTag->pos().x, fixedTag->pos().y);
                    QPoint tagPosition = QPoint(tag->pos().x, tag->pos().y);
                    QPoint deltaPosition = fixedTagPosition-tagPosition;
                    tagMovement += (deltaPosition * (1/length(deltaSignalStrength))) / _tagweight;
                }
            }
        }
    }
}
