#include "proximitydatamodel.h"

#include <QSettings>
#include <QTextStream>
#include <QHostAddress>


ProximityDataModel::ProximityDataModel()
{
    _contacts = new QHash<PTNP::TagContactKey, PTNP::OuttimingTagContact*>();
    _tags = new QMap<PTNP::oid, PTNP::LocatableProximityTag*>();
    _activatorTags = new QMap<PTNP::oid, PTNP::LocatableProximityTag*>();
    _antennas = new QMap<PTNP::aid, PTNP::ProximityAntenna*>();
    _buttons = new QMap<PTNP::oid, PTNP::ButtonEvent*>();
    _fixedTags = new QMap<PTNP::oid, PTNP::LocatableProximityTag*>();
}

ProximityDataModel::~ProximityDataModel()
{
    delete _contacts;
    delete _tags;
    delete _activatorTags;
    delete _antennas;
    delete _buttons;
}

void ProximityDataModel::readFromSettings() {
    QSettings settings; // alles aus der ini auslesen ...
                        // die muss natürlich an der entsprechenden Stelle im Dateisystem liegen
                        // näheres: http://doc.trolltech.com/4.6/qsettings.html#platform-specific-notes
                        // und alle Antennen enthalten ...
#ifdef _DEV_
    QTextStream cout(stdout, QIODevice::WriteOnly);
    cout<<"pdm.cpp: reading settings...\n";
#endif

    PTNP::position p;
    bool ok;
    quint32 r;

    int size = settings.beginReadArray("referencetags");
#ifdef _DEV_
    cout<<"pdm.cpp: reading "<<size<<" referencetags...\n";
#endif
    for(int i=0; i<size; ++i) {
        settings.setArrayIndex(i);
        PTNP::oid settingsOid = settings.value("id", "0").toUInt(&ok);
        if(ok) {
            r = settings.value("x",0).toUInt(&ok);
            if(ok)
                p.x = r;
            r = settings.value("y", 0).toUInt(&ok);
            if(ok)
                p.y = r;
            r = settings.value("floor", 0).toUInt(&ok);
            if(ok)
                p.f = r;
            PTNP::LocatableProximityTag* tag = new PTNP::LocatableProximityTag(settingsOid);
            tag->setPosition(p);
            tag->setFixed(true);
#ifdef _DEV_
            cout<<"pdm.cpp: referenctag "<<settingsOid<<"; position: "<<p.x<<" "<<p.y<<" "<<p.f<<"\n";
#endif
            _tags->insert(settingsOid, tag); // vielleicht überflüssig? Designentscheidung...
            _fixedTags->insert(settingsOid, tag);
        }
    }
    settings.endArray();

    size = settings.beginReadArray("antennas");
#ifdef _DEV_
    cout<<"pdm.cpp: reading "<<size<<" antennas...\n";
#endif
    for(int i=0; i<size; ++i) {
        settings.setArrayIndex(i);
        QString s = settings.value("ip", "0.0.0.0").toString();
        QHostAddress ha;
        ha.setAddress(s);
        quint32 qua = ha.toIPv4Address();
        PTNP::aid settingsAid = qua & 0x000000ff;
        r = settings.value("x", 0).toUInt(&ok);
        if(ok)
            p.x = r;
        r = settings.value("y", 0).toUInt(&ok);
        if(ok)
            p.y = r;
        r = settings.value("floor", 0).toUInt(&ok);
        if(ok)
            p.f = r;
        PTNP::ProximityAntenna* antenna = new PTNP::ProximityAntenna(settingsAid);
        antenna->setPosition(p);
#ifdef _DEV_
            cout<<"pdm.cpp: antenna "<<settingsAid<<"; position: "<<p.x<<" "<<p.y<<" "<<p.f<<"\n";
#endif
        _antennas->insert(settingsAid, antenna);
    }
    settings.endArray();
}
