#include "proximitydistributor.h"

#define MYVERSION QDataStream::Qt_4_4
#define MYBYTEORDER QDataStream::BigEndian

#ifdef _DEV_
#include <QHostAddress>
#endif

/*!
  Dieser Konstruktor übernimmt und initalisiert alle relevanten Werte.
  */
ProximityDistributor::ProximityDistributor(QList<ProximityServerClient *> &clients,
                                           QMutex &clientListMutex,
                                           QQueue<PTNP::Package*> &packageQueue,
                                           QMutex &packageQueueMutex,
                                           ProximityDataModel & model)
    : _clients(clients),
    _clientListMutex(clientListMutex),
    _packageQueue(packageQueue),
    _packageQueueMutex(packageQueueMutex),
    _model(model)
{
#ifdef _DEV_
    QTextStream cout(stdout, QIODevice::WriteOnly);
    cout<<"pd.cpp: init ProximityDistributor\n"<<flush;
#endif
}

/*!
  löscht alle temporären Objekte
  */
ProximityDistributor::~ProximityDistributor() {

}

/*!
  Diese Methode verteilt Pakete aus der Warteschlange an die Clients, die auf Pakete des jeweiligen Typs warten
  */
void ProximityDistributor::distribute()
{
    if(!_packageQueue.empty()) { // wenn neue Pakete vorhanden sind

        PTNP::Package* curPackage;

        // Paket holen
        _packageQueueMutex.lock();
        curPackage = _packageQueue.dequeue();
        _packageQueueMutex.unlock();

        send(curPackage);
        delete curPackage;
    }
}

/*!
  Veranlasst das Verschicken eines AllAntennasAndFixedTags-Pakets.
  Paket wird über send verschickt.

  */
void ProximityDistributor::distributeAllAntennasAndFixedTags()
{
    PTNP::Package* package;
    QByteArray* packageData;
    // Paket zusammensetzen
    packageData = new QByteArray();
    QDataStream packageDataStream(packageData, QIODevice::WriteOnly);

    packageDataStream<<(quint8)_model.antennas().count();
    packageDataStream<<(quint8)_model.fixedTags().count();
    QList<PTNP::aid> keys = _model.antennas().uniqueKeys();
    for(int i=0; i<_model.antennas().count(); i++) {
        packageDataStream << _model.antennas().value(keys.at(i))->aid();
        packageDataStream << _model.antennas().value(keys.at(i))->flags();
    }
    QList<PTNP::oid> fkeys = _model.fixedTags().uniqueKeys();
    for(int i=0; i<_model.fixedTags().count(); i++)
    {
        packageDataStream << _model.fixedTags().value(fkeys.at(i))->oid();
    }
    package = new PTNP::Package(PTNP::ALL_ANTENNAS_FIXED_TAGS, packageData);

    send(package);
    delete package;
}

/*!
  Diese Methode veranlasst das verschicken des übergebenen
  Pakets an alle Clients, die für diesen Pakettypen angemeldet sind.
  \param paket das zu verschickende Paket
  */
void ProximityDistributor::send(PTNP::Package* paket)
{
    QByteArray block;
    QDataStream out(&block, QIODevice::WriteOnly);
#ifdef _DEV_
    QTextStream cout(stdout, QIODevice::WriteOnly);
    cout<<"pd.cpp: versende ein ";
    switch((quint8)paket->header()){
    case PTNP::ALL_ANTENNAS_FIXED_TAGS:
        cout<<"ALL_ANTENNAS_FIXED_TAGS";
        break;
    case PTNP::BEACON_MSG:
        cout<<"BEACON_MSG";
        break;
    case PTNP::BUTTON_MSG:
        cout<<"BUTTON_MSG";
        break;
    case PTNP::TAG_TO_TAG_CONTACT_NEWS:
        cout<<"TAG_TO_TAG_CONTACT_NEWS";
        break;
    case PTNP::TAG_TO_TAG_CONTACT_RUNNING:
        cout<<"TAG_TO_TAG_CONTACT_RUNNING";
        break;
    case PTNP::POSITION_MSG:
        cout<<"POSITION_MSG";
        break;
    default:
        cout<<"unbekanntes";
    }
    cout<<" Paket an ";
#endif

    out.setVersion(MYVERSION);
    out.setByteOrder(MYBYTEORDER);

    // dann wird der Header in den block geschrieben
    out << (quint8)paket->header();
    out.writeRawData(paket->data()->data(), paket->data()->size());

    _clientListMutex.lock();
    if(!_clients.empty()) { // wenn es überhaupt empfangende Clients gibt
        QTcpSocket *clientSocket;
        ProximityServerClient* client;
        QList<ProximityServerClient*>::iterator iter;
        iter = _clients.begin();

        while (iter!=_clients.end()) {
            clientSocket = (*iter)->serverSocket();
            client = (*iter);
            if(clientSocket->state() != QAbstractSocket::ClosingState && clientSocket->state() != QAbstractSocket::UnconnectedState) {
                if(client->msgType() & paket->header()) { // wenn Client Nachrichten dieses Typs erwartet
#ifdef _DEV_
                    cout<<" client "<<clientSocket->peerAddress().toString()<<"\n";
                    cout<<"    Daten: ";
                    for(int i=0; i<block.size(); i++) {
                        hex(cout)<<(quint8)block.at(i)<<" "<<flush;
                    }
                    cout<<"\n"<<flush;
#endif
                    clientSocket->write(block);
                }
            }
            ++iter;
        }
        // hier müsste wohl noch das Multicast-zeugs rein
    }
#ifdef _DEV_
    else {
        cout<<" niemanden";
    }
    cout<<"\n"<<flush;
#endif
    _clientListMutex.unlock();
}
