#include "proximitylogger.h"

#include <QDateTime>
#include <QDataStream>
#include <QTextStream>

/*!
  errechnet die Zeitdifferenz in millisekunden zwischen zwei Zeitpunkten one und other
  \param one Startzeitpunkt
  \param other Endzeitpunkt
  */
static qint64 diffInMSecs(QDateTime one, QDateTime other)
{
    qint64 retVal = one.date().daysTo(other.date());
    retVal*=1000;
    retVal*=3600;
    retVal*=24;
    retVal+=one.time().msecsTo(other.time());
    return retVal;
}

/*!
  Dieser Konstruktor erstellt einen Logger und setzt das logfile auf 0.
  */
ProximityLogger::ProximityLogger()
{
    logFile = 0;
}

/*!
  Der Destruktor schließt das logfile, wenn es nicht 0 ist und löscht die Referenz.
  */
ProximityLogger::~ProximityLogger()
{
    if(logFile!= 0) {
        if(logFile->isOpen()) {
            logFile->close();
        }
        delete logFile;
    }
#ifdef _DEV_
    QTextStream cout(stdout, QIODevice::WriteOnly);
    cout<<"pl.cpp: destruktor!!!!\n"<<flush;
#endif
}

/*!
  Schreibt das übergebene Paket in eine zuvor geöffnete Logdatei.
  Dazu wird zunächst der Abstand in ms zum vorherigen Paket errechnet (64 Bit).
  Dieser wird in die log-Datei geschrieben und anschließend das Paket.
  Da das Paket eine immer feste Länge hat, kann im Anschluss alles sequenziell
  ausgelesen werden.
  \param entry das neue Paket
  */
void ProximityLogger::log(QByteArray entry)
{
#ifdef _DEV_
    QTextStream cout(stdout, QIODevice::WriteOnly);
    cout<<"pl.cpp: logge "<<entry.length()<<" "<<logFile->isOpen()<<"\n"<<flush;
#endif
    logFile->open(QIODevice::WriteOnly|QIODevice::Append);
    QDataStream out(logFile);
    QDateTime now(QDateTime::currentDateTime());
    // immer 64 Bit signed integer millisekunden seit logstart bis jetzt
    char* data;
    qint64 diff = diffInMSecs(lastEntry, now);
    data = (char*) &diff;
    //ret = logFile->write(data, sizeof(qint64));
    out.writeRawData(data, sizeof(qint64));
    data = entry.data();
    //ret = logFile->write(data, entry.size());
    out.writeRawData(data, entry.size());
#ifdef _DEV_
    cout<<"pl.cpp: Daten: ";
    hex(cout)<<diff<<" "<<flush;
    for(int i=0; i<entry.count(); i++)
        hex(cout)<<(quint8)entry.at(i)<<" "<<flush;
#endif
    logFile->close();
    lastEntry = QDateTime::currentDateTime();
}

/*!
  Erstellt aus dem aktuellen Zeitstempel ein logfile im aktuellen Ordner.
  Wenn das fehlschlägt, gibt diese Funktion \a false zurück, andernfalls \a true.
  Die erstellte Datei einen Dateinamen in der Form
    log-<monat><tag><stunde><minute><sekunde>.bin
  .
  */
bool ProximityLogger::createLogFile()
{
#ifdef _DEV_
    QTextStream cout(stdout, QIODevice::WriteOnly);
    cout<<"pl.cpp: erstelle Logdatei!\n"<<flush;
#endif
    lastEntry = QDateTime::currentDateTime();
    logFile = new QFile(tr("log-%1%2-%3:%4:%5.bin")
                        .arg(lastEntry.date().month())
                        .arg(lastEntry.date().day())
                        .arg(lastEntry.time().hour())
                        .arg(lastEntry.time().minute())
                        .arg(lastEntry.time().second()));
    return logFile->open(QIODevice::WriteOnly|QIODevice::Append);
}
