#include "proximityplayer.h"
#include "../common/protocol/openbeacon.h"

#include <QDataStream>
#ifdef _DEV_
#include <QTextStream>
#endif
#include <QtDebug>
#include <QTimer>
#include <QSettings>

ProximityPlayer::ProximityPlayer(QString path){
    this->initPlayer(path,false);
}

/*!
  Dieser Konstruktor erstellt einen Player anhand eines Logfiles \a path.
  */
ProximityPlayer::ProximityPlayer(QString path,bool playloop)
{
    this->initPlayer(path,playloop);
}

void ProximityPlayer::initPlayer(QString path,bool playloop){
#ifdef _DEV_
    QTextStream cout(stdout, QIODevice::WriteOnly);
    cout<<"pp.cpp: init ProximityPlayer"<<flush;
#endif
    _logFile = new QFile(path);
    _playLoop = playloop;
    QTimer::singleShot(0, this, SLOT(readData()));
    QSettings settings;
    _speed = settings.value("playbackspeed", "1").toUInt();
#ifdef _DEV_
    cout<<"pp.cpp: playbackspeed: "<<_speed<<endl;
#endif
    if(!openLogFile()) {
#ifdef _DEV_
        cout<<"konnte Logdatei nicht oeffnen!\n"<<flush;
#endif
        exit(4);
    }
#ifdef _DEV_
    cout<<"\n"<<flush;
#endif
}

/*!
  Dieser Destruktor schließt das logfile, wenn es geöffnet wurde und löscht alle Referenzen.
  */
ProximityPlayer::~ProximityPlayer()
{
    if(_logFile!= 0) {
        if(_logFile->isOpen()) {
            _logFile->close();
        }
    }
    delete _logFile;
}

/*!
  Diese Methode öffnet das logfile, welches dem Konstruktor als Pfad übergeben wurde.
  Schlägt das fehl, gibt diese Methode \a false zurück, andernfalls \a true.
  */
bool ProximityPlayer::openLogFile()
{
    if(!_logFile->open(QIODevice::ReadOnly)) {
        return false;
    }
    readData();
    return true;
}

/*!
  Diese Methode liest ein neues Paket aus der Datei und emitiert
   \sa readyRead() mit dem gelesenen Paket.
  Anschließend wird ein Timer gestartet, der nach der angegebenen
  Anzahl an ms wieder diese Methode aufruft, um das neue Paket zu lesen.
  Dabei ist das Timeout auch noch abhängig vom gewählten Playbackspeed
  */
void ProximityPlayer::readData()
{
    QByteArray* buffer = new QByteArray();
    QDataStream in(_logFile);
    qint64 timeout;
    int size = sizeof(TBeaconPackage);
    char* data;
    // immer 64 Bit signed integer millisekunden seit letzt gelesenen Paket
    in.readRawData((char*)&timeout, sizeof(qint64));
    data = new char[size];
    in.readRawData(data, size);
    buffer->append(data, size);
    delete []data;

    if(this->_logFile->atEnd() && this->_playLoop){
            this->_logFile->seek(0);
    }

    QTimer::singleShot(timeout/_speed, this, SLOT(readData()));
    emit readyRead(buffer, true);
}
