#include "proximityserialdataprovider.h"
//#ifndef Q_WS_MAC
#include <QApplication>
#include <QDebug>

#ifdef _DEV_
#include <QTextStream>
#endif

#include "../common/protocol/openbeacon.h"

/*!
  Dieser Konstruktor öffnet die serielle Schnittstelle.
  Schlägt das fehl, wird das Programm mit Fehlermeldung beendet.
  */
ProximitySerialDataProvider::ProximitySerialDataProvider(QString port)
{
    if(port=="")
        port=PORTNAME;
    _sp = new AbstractSerial();
    connect(_sp, SIGNAL(readyRead()), this, SLOT(readData()));
    _sp->setDeviceName(port);
    _sp->setBaudRate(AbstractSerial::BaudRate115200);
    qDebug() <<"psdpn.cpp: oeffne "<<port;
    if(!_sp->open(QIODevice::ReadOnly | QIODevice::Unbuffered)) {
        QTextStream cerr(stderr, QIODevice::WriteOnly);
        cerr << "psdpn.cpp: konnte serielle Schnitstelle "<<port<<" nicht oeffnen: "<<_sp->errorString()<<endl;
        qApp->exit(4);
    }
    connect(qApp, SIGNAL(aboutToQuit()), this, SLOT(deleteLater()));
    qDebug()<<"psdpn.cpp: erfolg!";
    // Isolator müssen 4 Byte sein - das ist die IP4-Source-Adresse
    _isolator.append(char(0x00));
    _isolator.append(char(0xff));
    _isolator.append(char(0x00));
    _isolator.append(char(0xff));
}

/*!
  Der Destruktor schließt den seriellen Port
  */
ProximitySerialDataProvider::~ProximitySerialDataProvider() {
    qDebug()<<"psdpn.cpp: schliesse seriellen Port";
    _sp->close();
    delete _sp;
}


/*!
  Diese Methode liest die Daten von der seriellen Schnittstelle und
  emitiert das readyRead-Signal mit dem gelesenen Paket
  */
void ProximitySerialDataProvider::readData() {
#ifdef _DEV_
    QTextStream cout(stdout, QIODevice::WriteOnly);
#endif
    while(_sp->bytesAvailable()) {
        QByteArray ba =_sp->read(sizeof(TBeaconEnvelope)+_isolator.count());
#ifdef _DEV_
        cout << "psdpn.cpp: Rx: ";
        for(int i=0; i<ba.size(); i++) {
            hex(cout)<<(quint8)ba.at(i)<<" "<<flush;
        }
#endif
        QByteArray* buffer = new QByteArray();
        buffer->append(ba.mid(_isolator.count(),sizeof(TBeaconEnvelope)));
        buffer->append(_isolator);
        emit(readyRead(buffer, true));
    }
}

//#endif
