#include "proximityserverclient.h"
#include "proximitydistributor.h"

#define MYVERSION QDataStream::Qt_4_4
#define MYBYTEORDER QDataStream::BigEndian

/*!
    hier wird ein neuer Client erzeugt mit der entsprechenden Socketverbindung zum Client
  */
ProximityServerClient::ProximityServerClient(QTcpSocket* socket,
                                             ProximityDataReceiver &receiver,
                                             ProximityDataModel &model)
    : _dataReceiver(receiver),
    _model(model)
{
    _socket = socket;
    _msgType = 0;
    connect(_socket, SIGNAL(readyRead()), this, SLOT(readControl()));
    connect(_socket, SIGNAL(disconnected()), this, SLOT(closeConnection()));
}

/*!
    der Destruktor tut nichts weiter
  */
ProximityServerClient::~ProximityServerClient() {
}


void ProximityServerClient::setMsgType(quint8 msgType) {
    _msgType = msgType;
}


/*!
  schickt eine Nachricht mit allen Antennen und fixierten Tags an den Client
  */
void ProximityServerClient::respondAllAntennas()
{
    QList<PTNP::aid> antennaKeys = _model.antennas().uniqueKeys();
    QList<PTNP::oid> fixedTagsKeys = _model.fixedTags().uniqueKeys();
    QByteArray block;
    // TODO: Alle Antennen zurueckschicken
    QDataStream out(&block, QIODevice::WriteOnly);
    out.setVersion(MYVERSION);
    out.setByteOrder(MYBYTEORDER);
    quint8 msgHeader;
    quint8 numAntennas = antennaKeys.count();
    quint8 numFixedTags = fixedTagsKeys.count();
    msgHeader = PTNP::ALL_ANTENNAS_FIXED_TAGS;

    // dann wird der Header gesendet
    out << msgHeader;
    // ab hier die Daten
    out << numAntennas;
    out << numFixedTags;
    for(int i=0; i<numAntennas; i++) {
        out << _model.antennas().value(antennaKeys.at(i))->aid();
        out << _model.antennas().value(antennaKeys.at(i))->flags();
    }
    for(int i=0; i<numFixedTags; i++) {
        out << _model.fixedTags().value(fixedTagsKeys.at(i))->oid();
    }

    _socket->write(block);
}

/*!
  liest eine ControllNachricht vom Client ein und wertet sie aus
  */
void ProximityServerClient::readControl()
{
#ifdef _DEV_
    QTextStream cout(stdout, QIODevice::WriteOnly);
#endif
    // der ganze Block ist ist wohl sinnlos
    // ==========================================================
    QTextStream cerr(stderr, QIODevice::WriteOnly);
    if(_socket->state() != QAbstractSocket::ClosingState && _socket->state() != QAbstractSocket::UnconnectedState) {
        if(_socket->bytesAvailable() < (int)sizeof(quint8)) { // erst Größe einer Controlmessage abwarten
            cerr<<"response-size reicht nicht!\t erwarte: " << (int)sizeof(quint8)<<" habe: "<<_socket->bytesAvailable()<<"\n"<<flush;
            _socket->disconnect();
            return;
        }
    // ==========================================================
        QDataStream in(_socket);

        in.setVersion(MYVERSION);
        in.setByteOrder(MYBYTEORDER);

        quint8 message;
        in >> message;

        quint8 msgType;
        msgType = message;

#ifdef _DEV_
        bin(cout)<<msgType;
        if(msgType & PTNP::ALL_ANTENNAS_FIXED_TAGS)
            cout<<" alle Antennen und fixierten Tags";
        if(msgType & PTNP::BEACON_MSG)
            cout<<" alle BeaconMessages";
        if(msgType & PTNP::BUTTON_MSG)
            cout<<" alle ButtonMessages";
        if(msgType & PTNP::TAG_TO_TAG_CONTACT_NEWS)
            cout<<" alle Tag contact news";
        if(msgType & PTNP::TAG_TO_TAG_CONTACT_RUNNING)
            cout<<" alle laufenden contact news";
        if(msgType & PTNP::POSITION_MSG)
            cout<<" alle PositionMessages";
#endif
        if(msgType & PTNP::ALL_ANTENNAS_FIXED_TAGS)
        {
            respondAllAntennas();
            // wenn neue Antennen gefunden werden, soll dieser Client benachrichtigt werden
            connect(&_dataReceiver, SIGNAL(newAntennaFound()), this, SLOT(respondAllAntennas()));
        }
        setMsgType(msgType);
#ifdef _DEV_
        cout<<"wohl erfolgreich!\n"<<flush;
#endif
    //===================== ebenso der hier: ==============
    }
#ifdef _DEV_
    else
        cout<<"nodata"<<flush;
#endif
    //=====================================================
}

/*!
  Diese Methode behandelt sämtliche Fehler, die am Socket auftreten können.
  \param e der Socketfehler
  */
void ProximityServerClient::handleSocketError(QAbstractSocket::SocketError e)
{
    QTextStream cerr(stderr, QIODevice::WriteOnly);
    cerr << "ERROR: ";
    switch(e) {
        case QAbstractSocket::ConnectionRefusedError:
            cerr << "connection refused!\n";
            break;
        case QAbstractSocket::RemoteHostClosedError:
            cerr << "the remote host closed the connection!\n";
            break;
        case QAbstractSocket::HostNotFoundError:
            cerr << "host not found!\n";
            break;
        case QAbstractSocket::SocketAccessError:
            cerr << "socket operation failed! Not enough privileges?\n";
            break;
        case QAbstractSocket::SocketResourceError:
            cerr << "local system ran out of resources!\n";
            break;
        case QAbstractSocket::SocketTimeoutError:
            cerr << "the socket operation timed out!\n";
            break;
        case QAbstractSocket::DatagramTooLargeError:
            cerr << "datagram was larger than the operating system's limit!\n";
            break;
        case QAbstractSocket::NetworkError:
            cerr << "an error occured with the network!\n";
            break;
        case QAbstractSocket::AddressInUseError:
            cerr << "the address specified is already in use and was set to be exclusive!\n";
            break;
        case QAbstractSocket::SocketAddressNotAvailableError:
            cerr << "the address specified does not belong to the host!\n";
            break;
        case QAbstractSocket::UnsupportedSocketOperationError:
            cerr << "the requested socket operation is not supportet by the local operating system!\n";
            break;
//        case QAbstractSocket::ProxyAuthenticationRequiredError:
//            cerr << "the socket is using a proxy, and the proxy requires authentication!\n";
//            break;
//        case QAbstractSocket::SslHandshakeFailedError:
//            cerr << "the SSL/TLS handshake failed, so the connection was closed!\n";
//            break;
//        case QAbstractSocket::UnfinishedSocketOperationError:
//            break;
//        case QAbstractSocket::ProxyConnectionRefusedError:
//            cerr << "could not contact the proxy server because the connection to that server was denied!\n";
//            break;
//        case QAbstractSocket::ProxyConnectionClosedError:
//            cerr << "the connection to the proxy server was closed unexpectedly!\n";
//            break;
//        case QAbstractSocket::ProxyConnectionTimeoutError:
//            cerr << "the connection to the proxy server timed out or the proxy server stopped responding in the authentication phase.\n";
//            break;
//        case QAbstractSocket::ProxyNotFoundError:
//            cerr << "the proxy address was not found!\n";
//            break;
//        case QAbstractSocket::ProxyProtocolError:
//            cerr << "the connection negotiation with the proxy server because the response from the proxy server could not be understood!\n";
//            break;
        case QAbstractSocket::UnknownSocketError:
            cerr << "an unidentified error occured.\n";
            break;
        default:
            cerr << "something somewhere went terribly wrong!\n";
    }
    cerr << flush;
}


