#ifndef PROXIMITYSERVERCLIENT_H
#define PROXIMITYSERVERCLIENT_H

#include <QObject>
#include <QMutex>
#include <QQueue>
#include <QTcpSocket>

#include "proximitydatamodel.h"
#include "proximitydatareceiver.h"

#include "../common/protocol/package.h"


//! Das ist der Client, wie er auf Serverseite existiert
/*!
  Der Client wird auf Serverseite mit einem zusätzlichen ControllReceiver-Thread ausgestattet,
  damit der Client im laufendem Betrieb beispielsweise alle Antennen abfragen kann.
  */
class ProximityServerClient : public QObject
{
    Q_OBJECT
public:
    //! Der Client auf Serverseite braucht immer einen socket
    ProximityServerClient(QTcpSocket* socket,
                          ProximityDataReceiver &receiver,
                          ProximityDataModel &model);
    //! tut momentan nichts
    ~ProximityServerClient();

    //! liefert den gewünschten Nachrichtentyp zurück
    PTNP::msgType msgType() { return _msgType; }
    //! setzt den Nachrichtentyp neu
    void setMsgType(PTNP::msgType msgType);

    //! gibt den Serversocket zurück
    QTcpSocket* serverSocket() const {return _socket;}

    //! vergleicht den aktuellen key mit dem übergebenem anhand der oid und sequenznummer
    inline bool operator==(const ProximityServerClient &psc2)
    {
        return this->_socket == psc2._socket;
    }
signals:
    //! wird emitiert, wenn der socket nicht mehr erreichbar ist
    void disconnected();
public slots:
    //! schickt alle Antennen an den Client
    void respondAllAntennas();
protected slots:
    //! dieser Slot teilt mit, dass der Socket die Verbindung verloren hat
    void closeConnection(){emit disconnected();};
    //! diese Methode liest eine Kontrollnachricht ein und beantwortet sie
    void readControl();
    //! diese Methode fängt alle Socketfehler ab
    void handleSocketError(QAbstractSocket::SocketError e);

private:
    ProximityDataReceiver &_dataReceiver;
    ProximityDataModel &_model;
    QTcpSocket* _socket; /*!< die Verbindung zum Client */
    PTNP::msgType _msgType; /*!< Der Nachrichtentyp, den dieser Client empfangen will */
};

#endif // PROXIMITYSERVERCLIENT_H
