#ifndef BEACONKEY_H
#define BEACONKEY_H

#include <QObject>

namespace PTNP {
    /*!
      Diese Klasse wird genutzt, um die Hashtabelle zu verwalten, auf die der Zugriff per oid und sequenznummer geschieht.
      Also um zu prüfen, ob ein Beacon von verschiedenen Antennen gleichzeitig gemeldet wurde.
      */
    class BeaconKey : public QObject
    {
    public:
        //! der Standardkonstruktor mit der oid und Sequenznummer des Tags
        BeaconKey(const quint16 oid, const quint32 seq): _oid(oid), _seq(seq){};

        //! gibt die oid des Tags zurück
        quint16 oid() const {return _oid;}
        //! gibt die Sequenznummer des Tags zurück
        quint32 seq() const {return _seq;}

    private:
        quint16 _oid;
        quint32 _seq;
    };

    //! vergleicht den aktuellen key mit dem übergebenem anhand der oid und sequenznummer
    inline bool operator==(const BeaconKey &bk1, const BeaconKey &bk2)
    {
        return bk1.oid() == bk2.oid()
           && bk1.seq() == bk2.seq();
    }
    //! errechnet den Hashkey vom übergebenen BeaconKey
    inline uint qHash(const BeaconKey &key)
    {
        // XORt zweimal die oid nebeneinander (macht ja dann 32 Bit) mit der sequenznummer (die eh schon 32 Bit breit ist)
        return ((key.oid()<<16)|key.oid())^key.seq();
    }
}

#endif // BEACONKEY_H
