#include "locatableproximitytag.h"

#ifdef _DEV_
#include <QTextStream>
#endif

#include "outtimingsignalstrength.h"
#include "outtimingtagcontact.h"
#include "proximitytag.h"

#include <QSettings>

namespace PTNP {
    LocatableProximityTag::LocatableProximityTag(PTNP::oid o, QObject* parent, bool fixed) : OuttimingProximityTag(o, parent, fixed)
    {
        QSettings settings;
        _numSignalStrength = settings.beginReadArray("signalstrengthweights");
        settings.endArray();
    }

    LocatableProximityTag::~LocatableProximityTag()
    {
    }

    template<typename T>
    quint16 LocatableProximityTag::getSignalStrengthToT(T a, QMap<T, QVector<int> > map) {
        quint16 retVal = 0; // die Anfangsdistanz ist 0
        quint16 maxVal = 0;
        quint16 maxPaketsPerSignalStrength = 0;
        if(map.contains(a)) {
            QVector<int> v = map.value(a); // Signalstärkevektor dieser Antenne holen
            // alle Signalstärkepakete gewichtet nach signalStrengthWeight zusammenaddieren
            for(int i=0; i<_numSignalStrength; i++) {
                maxPaketsPerSignalStrength += v[i];
                retVal+=v[i]*signalStrengthWeight[i];
            }
            maxPaketsPerSignalStrength = maxPaketsPerSignalStrength / _numSignalStrength;
            if(maxPaketsPerSignalStrength>=1) {
                for(int i=0; i<_numSignalStrength; i++) {
                    maxVal += maxPaketsPerSignalStrength*signalStrengthWeight[i];
                }
#ifdef _DEV_
                QTextStream cout(stdout, QIODevice::WriteOnly);
                cout<<"lpt.cpp id: "<<oid()<<" maxVal: "<<maxVal<<" retVal: "<<retVal<<" und daraus wird: "<<(quint16)((retVal*100) / maxVal)<<"% Bewegung"<<endl;
#endif
                return (quint16) ((retVal * 100) / maxVal); // prozentuale Anziehungskraft
            }
        }
        return 0;
    }

    template<typename T>
    QMap<T, quint16> LocatableProximityTag::getAttractionToT(QMap<T, QVector<int> > map) {
        QMap<T, quint16> retVal;
        QList<T> keys = map.keys();
        // alle Distanzen zu allen Antennen berechnen, indem über die Liste vorhandener Schlüssel iteriert wird
        for(int i=0; i<keys.length(); i++) {
            retVal.insert(keys.at(i), getSignalStrengthToT<T>(keys.at(i), map));
        }
        return retVal;
    }

    /*!
      Diese Methode liefert alle Distanzen / Anziehungskräfte auf dieses Tag zurück
      */
    QMap<PTNP::aid, quint16> LocatableProximityTag::getAntennaAttractions()
    {
        return getAttractionToT<PTNP::aid>(_strengthPerAntenna);
    }

    /*!
      Diese Methode liefert alle Distanzen/ Anziehungskräfte zwischen diesem und den
      fixierten Tags zurück
      */
    QMap<PTNP::oid, quint16> LocatableProximityTag::getFixedTagAttractions()
    {
        return getAttractionToT<PTNP::oid>(_strengthPerContactedTag);
    }

    void LocatableProximityTag::seen(LocatableProximityTag& ot, PTNP::signalStrength s) {
        PTNP::oid o = ot.oid();
        if(!_strengthPerContactedTag.contains(o)) {
            QVector<int> v(_numSignalStrength);
            for(int i=0; i<_numSignalStrength; i++) {
                v[i] = 0;
            }
            _strengthPerContactedTag.insert(o, v);
        }
        _strengthPerContactedTag[o][s]+=1;
        PTNP::OuttimingSignalStrength* oss = new PTNP::OuttimingSignalStrength(o, s);
        connect(oss, SIGNAL(timeout()), this, SLOT(contactBeaconTimeout()));
        connect(oss, SIGNAL(timeout()), oss, SLOT(deleteLater()));
        oss->start();
    }

    /*!
      Dieser slot ist die Erweiterung des Slots ProximityTag::seen().
      Hier werden die Signalstärkepakete verarbeitet und timeouts für sie gestartet.
      */
    void LocatableProximityTag::seen(PTNP::aid a, PTNP::signalStrength s)
    {
        OuttimingProximityTag::seen();
        if(!_strengthPerAntenna.contains(a)) {
            QVector<int> v(_numSignalStrength);
            // initalisieren des Signalstärkevektors
            for(int i=0; i<_numSignalStrength; i++)
                v[i] = 0;
            _strengthPerAntenna.insert(a, v);
        }
        _strengthPerAntenna[a][s]+=1; // Anzahl der Beacons der Signalstärke s an der Antenne a hochzählen
        PTNP::OuttimingSignalStrength* oss = new PTNP::OuttimingSignalStrength(a, s);
#ifdef _DEV_
        QTextStream cout(stdout, QIODevice::WriteOnly);
        cout<<"lpt.cpp: antID: "<<oss->aid()<<"; strength: "<<oss->signalStrength()<<" increased to: "<<_strengthPerAntenna[oss->aid()][oss->signalStrength()]<<"\n"<<flush;
#endif
        connect(oss,SIGNAL(timeout()), this, SLOT(beaconTimeout()));
        connect(oss, SIGNAL(timeout()), oss, SLOT(deleteLater()));
        oss->start();
    }

    /*!
      Dieser Slot wird aufgerufen, wenn ein Kontakt austimt.
      Hier wird aus der Liste der Kontaktsignalstärken das kontaktierte Tag gelöscht.
      */
    void LocatableProximityTag::contactTimeout() {
        PTNP::OuttimingTagContact* otc = qobject_cast<PTNP::OuttimingTagContact*> (sender());
        const PTNP::ProximityTag* otherTag;
        if(otc->tag1() == (PTNP::ProximityTag*)this)
            otherTag = otc->tag2();
        else
            otherTag = otc->tag1();
        _strengthPerContactedTag.remove(otherTag->oid());
    }

    /*!
      Slot, der ausgeführt wird, sobald ein Beacon austimt
      */
    void LocatableProximityTag::beaconTimeout()
    {
        // Signalstärke und Antenne rausfinden, dessen Beacon ausgetimt hat
        PTNP::OuttimingSignalStrength* oss = qobject_cast<PTNP::OuttimingSignalStrength*>(sender());
        _strengthPerAntenna[oss->aid()][oss->signalStrength()]-=1; // Anzahl der Beacons dieser Signalstärke runterzählen
#ifdef _DEV_
        QTextStream cout(stdout, QIODevice::WriteOnly);
        cout<<"lpt.cpp: antID: "<<oss->aid()<<"; strength: "<<oss->signalStrength()<<" decreased to: "<<_strengthPerAntenna[oss->aid()][oss->signalStrength()]<<"\n"<<flush;
#endif
    }

    void LocatableProximityTag::contactBeaconTimeout()
    {
        PTNP::OuttimingSignalStrength* oss = qobject_cast<PTNP::OuttimingSignalStrength*>(sender());
        // das oss->aid() ist nicht schön, denn eigentlich ist es hier eine oid ... aber
        // ansonsten müsste ich die gesamte OuttimingSignalStrength-Klasse einmal kopieren
        // und nur aid gegen oid tauschen.
        _strengthPerContactedTag[oss->aid()][oss->signalStrength()]-=1;
    }
}
