#ifndef LOCATABLEPROXIMITYTAG_H
#define LOCATABLEPROXIMITYTAG_H

#include <QObject>
#include <QVector>
#include "proximityantenna.h"
#include "outtimingproximitytag.h"

namespace PTNP{
    /*!
      Diese Klasse beschreibt ein ortbared ProximityTag.
      */
    class LocatableProximityTag : public OuttimingProximityTag
    {
        Q_OBJECT
    public:
        //! Standardkonstruktor für ein ortbares ProximityTag
        LocatableProximityTag(PTNP::oid, QObject* parent=0, bool fixed=false);
        ~LocatableProximityTag();
        QMap<PTNP::aid, quint16> getAntennaAttractions();
        QMap<PTNP::oid, quint16> getFixedTagAttractions();
        //! Setzt den Zeitstempel neu und aktualisiert _strengthPerContactedTag
        void seen(LocatableProximityTag& ot, PTNP::signalStrength);
        //! Setzt den Zeitstempel neu und aktualisiert das _strengthPerAntennaFeld - timeout wird gestartet dieses ProximityTags
        void seen(PTNP::aid, PTNP::signalStrength);
    public slots:
        void contactTimeout();
    private slots:
        void beaconTimeout();
        void contactBeaconTimeout();
    private:
        template<typename T>
        quint16 getSignalStrengthToT(T, QMap<T, QVector<int> >);
        template<typename T>
        QMap<T, quint16> getAttractionToT(QMap<T, QVector<int> >);
        //! diese Map hält die Anzahl von Beacons pro Signalstärke für jede Antenne -> daraus errechnet sich die Distanz
        QMap<PTNP::aid, QVector<int> > _strengthPerAntenna;
        QMap<PTNP::oid, QVector<int> > _strengthPerContactedTag;
        quint8 _numSignalStrength;
    };
}
#endif // LOCATABLEPROXIMITYTAG_H
