#include "tagcontact.h"
#include <QDateTime>

namespace PTNP {
    /*!
      Dieser Konstruktor erstellt aus zwei Tags, einer Antenne und einer Signalstärke ein Kontaktobjekt.
      */
    TagContact::TagContact(ProximityTag* const tag1, ProximityTag* const tag2, ProximityAntenna* const antenna, PTNP::signalStrength strength)
        : tag1_(tag1), tag2_(tag2)
    {
        TagContactInfo* info = new TagContactInfo(antenna, strength);
        _timestamp_LastView = QDateTime::currentDateTime().toTime_t();
        _timestamp_FirstView = _timestamp_LastView;
        _contactInfo.append(info);
    }

    /*!
      löscht die Kontaktinformationen
      */
    TagContact::~TagContact()
    {
        while(!_contactInfo.isEmpty())
            delete _contactInfo.takeFirst();
    }

    bool TagContact::canBeIgnored() {
        if( (_ignoreCounter / _seenCounter) > IGNORETHRESHOLD)
            return true;
        return false;
    }

    /*!
      startet den Timeout neu und gibt auch den Tags die Nachricht, dass sie gesehen wurden
      */
    void TagContact::seen(ProximityAntenna* const antenna, PTNP::signalStrength strength)
    {
        _timestamp_LastView = QDateTime::currentDateTime().toTime_t();
        _actualDuration = _timestamp_LastView - _timestamp_FirstView;
        tag1_->seen();
        tag2_->seen();
        _seenCounter += 1;
        if(antenna->ignoreReportedContacts())
            _ignoreCounter += 1;
        TagContactInfo* tci = new TagContactInfo(antenna, strength);
        int i = _contactInfo.indexOf(tci);
        if(i != -1) { // wenn es eine TagContactInfo zu diesem Contact schon gibt, wird diese aktualisiert
            TagContactInfo* temp = _contactInfo.at(i);
            _contactInfo.removeAt(i);
            _contactInfo.prepend(temp);
        } else {
            _contactInfo.prepend(tci);
        }
    }

}
