#ifndef TAGCONTACTINFO_H
#define TAGCONTACTINFO_H

#include <QObject>

#include "proximityantenna.h"

namespace PTNP {

    /*!
      Objekte dieser Klasse enthalten alle Informationen, die zu Kontakten noch gespeichert werden sollten (außer den zwei TagIDs).
      So kann zum Beispiel aus der Signalstärke die relative Position zweier Tags untereinander ermittelt werden.
      */
    class TagContactInfo : public QObject
    {
    public:
        //! erstellt aus Antenne und Signalstärke zwischen zwei Tags eine Information
        TagContactInfo(ProximityAntenna* const antenna, quint8 strength);
        //! tut momentan nichts
        ~TagContactInfo(){};

        //! gibt die aktuelle Signalstärke zwischend en Tags zurück \returns strength
        quint8 strength() { return _strength; }
        //! setzt die Signalstärke neu
        // TODO: als SLOT implementieren und stärke aus mehreren Werten mitteln
        void setStrength(quint8 strength) { _strength = strength; }
        //! gibt die Antenne zurück, über die dieser Kontakt gesichtet wurde \returns antenna
        const ProximityAntenna* antenna() { return _antenna; }
        //! gibt zurück, wann der Kontakt zuletzt gesehen wurde
        QDateTime lastSeen() { return _lastSeen; }
        //! vergleicht zwei Kontaktinformationen und gibt \a true, wenn Operand 1 aktueller ist
        // wird vor allem für die Sortierung benötigt
        bool operator<(TagContactInfo const& tci2) const;
        //! gibt \a true zurück, wenn beide Kontakte von derselben Antenne registriert wurden
        bool operator==(TagContactInfo const& tci2) const;

    private:
        QDateTime _lastSeen; // nötig für einen Datenbankeintrag
        ProximityAntenna* const _antenna;
        quint8 _strength;
    };
}

#endif // TAGCONTACTINFO_H
