#ifndef TAGCONTACTKEY_H
#define TAGCONTACTKEY_H

#include <QObject>
#include "PTNP.h"

namespace PTNP {
    /*!
      Diese Klasse wird genutzt, um die Hashtabelle zu verwalten, auf die der Zugriff per oid1 und oid2 geschieht.
      Um also zu prüfen, ob man den Kontakt der beiden Tags schon registriert hat
      */
    class TagContactKey
    {
    public:
        //! der Standardkonstruktor mit den oids der Tags
        TagContactKey(const quint16 oid1, const quint16 oid2):_oid1(oid1), _oid2(oid2){};
        //! tut momentan nichts
        ~TagContactKey(){};

        //! gibt die oid des ersten Tags zurück
        oid oid1() const {return _oid1;}
        //! gibt die oid des zweiten Tags zurück
        oid oid2() const {return _oid2;}
        bool contains(oid o) const;

    private:
        oid _oid1;
        oid _oid2;
    };

    //! vergleicht den aktuellen Key mit einem anderen anhand der beiden OIDs
    inline bool operator==(const TagContactKey &tck1, const TagContactKey &tck2)
    {
        return (tck1.oid1() == tck2.oid1()
           && tck1.oid2() == tck2.oid2()) ||
           (tck1.oid1() == tck2.oid2()
           && tck1.oid2() == tck2.oid1());
    }
    //! errechnet den Hashkey vom übergebenen TagContactKey
    inline uint qHash(const TagContactKey &key)
    {
        return key.oid1()^key.oid2();
    }
}

#endif // TAGCONTACTKEY_H
